import re

def filterLines(lines, regex):
	'''Filters each line of the given line iterator using the given regular
	expression string. For each match, a tuple containing the text matching
	each capture group from the regular expression is yielded.
	'''
	matcher = re.compile(regex)
	for line in lines:
		if line.endswith('\n'):
			line = line[ : -1]
		match = matcher.match(line)
		if match:
			yield match.groups()

def filterFile(filePath, regex):
	'''Filters each line of the given text file using the given regular
	expression string. For each match, a tuple containing the text matching
	each capture group from the regular expression is yielded.
	'''
	inp = open(filePath, 'r')
	try:
		for groups in filterLines(inp, regex):
			yield groups
	finally:
		inp.close()

def joinContinuedLines(lines):
	'''Iterates through the given lines, replacing lines that are continued
	using a trailing backslash with a single line.
	'''
	buf = ''
	for line in lines:
		if line.endswith('\\\n'):
			buf += line[ : -2]
		elif line.endswith('\\'):
			buf += line[ : -1]
		else:
			yield buf + line
			buf = ''
	if buf:
		raise ValueError('Continuation on last line')

def extractMakeVariables(filePath):
	'''Extract all variable definitions from the given Makefile.
	Returns a dictionary that maps each variable name to its value.
	'''
	makeVars = {}
	inp = open(filePath, 'r')
	try:
		for name, value in filterLines(
			joinContinuedLines(inp),
			r'[ ]*([A-Za-z0-9_]+)[ ]*:=(.*)'
			):
			makeVars[name] = value.strip()
	finally:
		inp.close()
	return makeVars

def parseBool(valueStr):
	'''Parses a string containing a boolean value.
	Accepted values are "true" and "false"; anything else raises ValueError.
	'''
	if valueStr == 'true':
		return True
	elif valueStr == 'false':
		return False
	else:
		raise ValueError('Invalid boolean "%s"' % valueStr)



# Generates configuration headers for VC++ builds

import sys
import os.path
import outpututils
import win_resource
import version2code

#
# platform: one of { Win32, x64 }
# configuration: one of { Debug, Developer, Release }
# outputPath: the location in which to generate config files
#
def genConfig(platform, configuration, outputPath):
	
	#
	# resource-info.hh
	#
	resourceInfoHeader = os.path.join(outputPath, 'resource-info.h')
	generator = win_resource.iterResourceHeader()
	outpututils.rewriteIfChanged(resourceInfoHeader, generator)

	#
	# version.ii
	#
	versionHeader = os.path.join(outputPath, 'version.ii')
	generator = version2code.iterVersionInclude()
	outpututils.rewriteIfChanged(versionHeader, generator)

if len(sys.argv) == 4:
	genConfig(sys.argv[1], sys.argv[2], sys.argv[3])
else:
	print >> sys.stderr, 'Usage: python genconfig.py platform configuration outputPath'
	sys.exit(2)



import os, sys
import zipfile
import packagewindows

def AddFile(zip, path, zipPath):
	print 'Adding ' + path
	zip.write(path, zipPath, zipfile.ZIP_DEFLATED)

def AddDirectory(zip, root, zipPath):
	for path, dirs, files in os.walk(root):
		if '.svn' in dirs:
			dirs.remove('.svn')  # don't visit .svn directories
		for name in files:
			thisZipPath = zipPath
			if os.path.abspath(root) != os.path.abspath(path):
				thisZipPath = os.path.join(thisZipPath, os.path.relpath(path, root))
			AddFile(zip, os.path.join(path, name), os.path.join(thisZipPath, name))

def PackageZip(info):

	if not os.path.exists(info.packagePath):
		os.mkdir(info.packagePath)

	zipFileName = info.installerFileName + '.zip'
	zipFilePath = os.path.join(info.packagePath, zipFileName)
	if os.path.exists(zipFilePath):
		os.unlink(zipFilePath)

	print 'Generating ' + zipFilePath
	zip = zipfile.ZipFile(zipFilePath, 'w')
	AddFile(zip, os.path.join(info.installPath, 'openmsx-debugger.exe'), 'openmsx-debugger.exe')
	zip.close()
	
	zipFileName = info.installerFileName + '-pdb.zip'
	zipFilePath = os.path.join(info.packagePath, zipFileName)
	if os.path.exists(zipFilePath):
		os.unlink(zipFilePath)

	print 'Generating ' + zipFilePath
	zip = zipfile.ZipFile(zipFilePath, 'w')
	AddFile(zip, os.path.join(info.installPath, 'openmsx-debugger.pdb'), 'openmsx-debugger.pdb')
	zip.close()

if __name__ == '__main__':
	if len(sys.argv) != 4:
		print >> sys.stderr, 'Usage: python packagezip.py platform configuration version'
		# E.g. build\package-windows\package.cmd x64 Release 0.7.1
		sys.exit(2)
	else:
		info = packagewindows.PackageInfo(sys.argv[1], sys.argv[2], sys.argv[3], '')
		PackageZip(info)



#!/usr/bin/env python2
# Creates a source distribution package.

from os import makedirs, remove
from os.path import isdir
from subprocess import CalledProcessError, PIPE, Popen, check_output
from tarfile import TarError, TarFile
import stat, sys

verbose = False

def archiveFromGit(versionedPackageName, committish):
	prefix = '%s/' % versionedPackageName
	distBase = 'derived/dist/'
	umask = (
			stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR |
			stat.S_IRGRP |                stat.S_IXGRP |
			stat.S_IROTH |                stat.S_IXOTH
			)

	def exclude(info):
		'''Returns True iff the given tar entry should be excluded.
		'''
		return any((
				info.name.endswith('/.gitignore'),
				))

	proc = Popen(
			('git', 'archive', '--prefix=' + prefix, '--format=tar', committish),
			bufsize = -1,
			stdin = None, stdout = PIPE, stderr = sys.stderr,
			)
	try:
		outTarPath = distBase + versionedPackageName + '.tar.gz'
		print 'archive:', outTarPath
		if not isdir(distBase):
			makedirs(distBase)
		outTar = TarFile.open(outTarPath, 'w:gz')
		try:
			# Copy entries from "git archive" into output tarball, except for
			# excluded entries.
			numIncluded = numExcluded = 0
			inTar = TarFile.open(mode = 'r|', fileobj = proc.stdout)
			try:
				for info in inTar:
					if exclude(info):
						if verbose:
							print 'EX', info.name
						numExcluded += 1
					else:
						if verbose:
							print 'IN', info.name
						numIncluded += 1
						info.uid = info.gid = 1000
						info.uname = info.gname = 'openmsx'
						info.mode = info.mode & umask
						outTar.addfile(info, inTar.extractfile(info))
			finally:
				inTar.close()
			print 'entries: %d included, %d excluded' % (
					numIncluded, numExcluded)
		except:
			# Clean up partial output file.
			outTar.close()
			remove(outTarPath)
			raise
		else:
			outTar.close()
	except:
		proc.terminate()
		raise
	else:
		data, _ = proc.communicate()
		if len(data) != 0:
			print >> sys.stderr, (
					'WARNING: %d more bytes of data from "git archive" after '
					'tar stream ended' % len(data))

def niceVersionFromGitDescription(description):
	'''Our release tag names are still based on naming limitations from CVS;
	convert them to something more pleasing to the eyes.
	'''
	parts = description.split('-')
	if parts[0].startswith('RELEASE_'):
		tagParts = tag.split('_')[1 : ]
		i = 0
		while i < len(tagParts) and tagParts[i].isdigit():
			i += 1
		parts[0] = '-'.join(
				['.'.join(tagParts[ : i])] +
				[s.lower() for s in tagParts[i : ]]
				)

	if len(parts) >= 2:
		parts[1] = 'dev' + parts[1]

	if parts[0] == 'INIT':
		del parts[0]

	return '-'.join(parts)

def getDescription(committish):
	args = ['git', 'describe', '--abbrev=9']
	if committish is not None:
		args.append(committish)
	try:
		return check_output(args).rstrip('\n')
	except CalledProcessError as ex:
		print >> sys.stderr, '"%s" returned %d' % (
				' '.join(args), ex.returncode
				)
		raise

def main(committish = None):
	try:
		description = getDescription(committish)
	except CalledProcessError:
		sys.exit(1)
	version = niceVersionFromGitDescription(description)
	try:
		archiveFromGit('openmsx-debugger-%s' % version, description)
	except (OSError, TarError) as ex:
		print >> sys.stderr, 'ERROR: %s' % ex
		sys.exit(1)

if __name__ == '__main__':
	if len(sys.argv) == 1:
		main()
	elif len(sys.argv) == 2:
		main(sys.argv[1])
	else:
		print >> sys.stderr, 'Usage: gitdist.py [branch | tag]'
		sys.exit(2)



# Detect the native CPU and OS.
# Actually we rely on the Python "platform" module and map its output to names
# that the openMSX build understands.

from platform import machine, python_version, system
import sys

def detectCPU():
	'''Detects the CPU family (not the CPU model) of the machine were are
	running on.
	Raises ValueError if no known CPU is detected.
	'''
	cpu = machine().lower()
	dashIndex = cpu.find('-')
	if dashIndex != -1:
		# Hurd returns "cputype-cpusubtype" instead of just "cputype".
		cpu = cpu[ : dashIndex]
	if cpu in ('x86_64', 'amd64'):
		return 'x86_64'
	elif cpu in ('x86', 'i386', 'i486', 'i586', 'i686'):
		return 'x86'
	elif cpu.startswith('ppc') or cpu.endswith('ppc') or cpu.startswith('power'):
		return 'ppc64' if cpu.endswith('64') else 'ppc'
	elif cpu.startswith('arm'):
		return 'arm'
	elif cpu == 'aarch64':
		return 'aarch64'
	elif cpu == 'aarch64_be':
		return 'aarch64_be'
	elif cpu.startswith('mips') or cpu == 'sgi':
		return 'mipsel' if cpu.endswith('el') else 'mips'
	elif cpu == 'm68k':
		return 'm68k'
	elif cpu == 'ia64':
		return 'ia64'
	elif cpu.startswith('alpha'):
		return 'alpha'
	elif cpu.startswith('hppa') or cpu.startswith('parisc'):
		return 'hppa'
	elif cpu.startswith('s390'):
		return 's390'
	elif cpu.startswith('sparc') or cpu.startswith('sun4u'):
		return 'sparc'
	elif cpu.startswith('sh'):
		return 'sheb' if cpu.endswith('eb') else 'sh'
	elif cpu == 'avr32':
		return 'avr32'
	elif cpu == '':
		# Python couldn't figure it out.
		os = system().lower()
		if os == 'windows':
			# Relatively safe bet.
			return 'x86'
		raise ValueError('Unable to detect CPU')
	else:
		raise ValueError('Unsupported or unrecognised CPU "%s"' % cpu)

def detectOS():
	'''Detects the operating system of the machine were are running on.
	Raises ValueError if no known OS is detected.
	'''
	os = system().lower()
	if os in ('linux', 'darwin', 'freebsd', 'netbsd', 'openbsd', 'gnu'):
		return os
	elif os.startswith('gnu/'):
		# GNU userland on non-Hurd kernel, for example Debian GNU/kFreeBSD.
		# For openMSX the kernel is not really relevant, so treat it like
		# a generic GNU system.
		return 'gnu'
	elif os.startswith('mingw') or os == 'windows':
		return 'mingw32'
	elif os == 'sunos':
		return 'solaris'
	elif os == '':
		# Python couldn't figure it out.
		raise ValueError('Unable to detect OS')
	else:
		raise ValueError('Unsupported or unrecognised OS "%s"' % os)

if __name__ == '__main__':
	try:
		print >> sys.stderr, '  Using Python %s native system detection...' % (
			python_version()
			)
		hostCPU = detectCPU()
		hostOS = detectOS()
		if hostOS == 'mingw32' and hostCPU == 'x86_64':
			# It is possible to run MinGW on 64-bit Windows, but producing
			# 64-bit code is not supported yet.
			hostCPU = 'x86'
		print >> sys.stderr, '  Detected system: %s-%s' % (hostCPU, hostOS)
		print 'OPENMSX_TARGET_CPU=%s' % hostCPU
		print 'OPENMSX_TARGET_OS=%s' % hostOS
	except ValueError, ex:
		print >> sys.stderr, ex
		sys.exit(1)



import os, sys

def DeleteDirectoryIfExists(top):
	if os.path.exists(top):
		DeleteDirectory(top)
		
def DeleteDirectory(top):
	for root, dirs, files in os.walk(top, topdown=False):
		for name in files:
			os.remove(os.path.join(root, name))
		for name in dirs:
			os.rmdir(os.path.join(root, name))

def GenerateInstallFiles(info):
	DeleteDirectoryIfExists(info.makeInstallPath)
	install.installAll(info.makeInstallPath + os.sep, 'bin', 'share', 'doc', info.openmsxExePath, 'mingw32', True, True)

def WalkPath(sourcePath):
    if os.path.isfile(sourcePath):
        filenames = list()
        filenames.append(os.path.basename(sourcePath))
        yield os.path.dirname(sourcePath), list(), filenames
    else:
        for dirpath, dirnames, filenames in os.walk(sourcePath):
            yield dirpath, dirnames, filenames

class PackageInfo:

	def __init__(self, platform, configuration, version, catapultPath):
		
		self.platform = platform.lower()
		if self.platform == 'win32':
			self.architecture = 'x86'
			self.platform = 'Win32'
			self.win64 = False
		elif self.platform == 'x64':
			self.architecture = 'x64'
			self.platform = 'x64'
			self.win64 = True
		else:
			raise ValueError, 'Wrong platform: ' + platform
			
		self.configuration = configuration.lower()
		if self.configuration == 'release':
			self.configuration = 'Release'
			self.catapultConfiguration = 'Unicode Release'
		elif self.configuration == 'Developer':
			self.configuration = 'Developer'
			self.catapultConfiguration = 'Unicode Debug'
		elif self.configuration == 'Debug':
			self.configuration = 'Debug'
			self.catapultConfiguration = 'Unicode Debug'
		else:
			raise ValueError, 'Wrong configuration: ' + architecture

		self.version = version
		self.catapultPath = catapultPath
		
		# Useful variables
		self.buildFlavor = self.platform + '-VC-' + self.configuration
		self.buildPath = os.path.join('derived', self.buildFlavor)
		self.installPath = os.path.join(self.buildPath, 'install')
		self.sourcePath = 'src'
		self.codecPath = 'Contrib\\codec\\Win32'
		self.packageWindowsPath = 'build\\package-windows'
		
		self.packagePath = os.path.join(self.buildPath, 'package-windows')
		self.makeInstallPath = os.path.join(self.packagePath, 'install')
		
		self.installerFileName = 'openmsx-debugger-' + version + '-VC-' + self.architecture
		
if __name__ == '__main__':
	if len(sys.argv) == 5:
		info = PackageInfo(sys.argv[1], sys.argv[2], sys.argv[3], sys.argv[4])
	else:
		print >> sys.stderr, \
			'Usage: python packagewindowsinfo.py architecture, configuration, version, catapultPath'
		sys.exit(2)
